/**
 *
 * Copyright (c) 2010, Zed A. Shaw and Mongrel2 Project Contributors.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 * 
 *     * Neither the name of the Mongrel2 Project, Zed A. Shaw, nor the names
 *       of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written
 *       permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include "../commands.h"
#include <routing.h>
#include <dbg.h>

int Command_uuid(Command *cmd)
{
    return system("uuidgen");
}

static void null_destroy(Route *route, struct RouteMap *map)
{

}


int Command_route(Command *cmd)
{
    int rc = 0;
    RouteMap *map = RouteMap_create(null_destroy);
    check(map != NULL, "Error, can't create the RouteMap.");

    bstring match = option(cmd, "match", NULL);
    bstring pattern = option(cmd, "pattern", NULL);
    bstring reversed = option(cmd, "reversed", NULL);
    bstring sfxmatch = option(cmd, "suffix", NULL);

    check(match != NULL, "You have to give something to match against with --match.");
    check(pattern != NULL, "You have to give a pattern to use with --pattern.");

    if(reversed == NULL) {
        rc = RouteMap_insert(map, bstrcpy(pattern), NULL);
        check(rc == 0, "Failed to insert pattern into routing table.");
    } else {
        rc = RouteMap_insert_reversed(map, bstrcpy(pattern), NULL);
        check(rc == 0, "Failed to insert REVERSED pattern into routing table.");
    }

    Route *route = NULL;

    if(sfxmatch != NULL) {
        route = RouteMap_match_suffix(map, match);
    } else {
        route = RouteMap_simple_prefix_match(map, match);
    }

    if(route != NULL) {
        printf("Match passed on '%s' against '%s'.\n", bdata(match), bdata(pattern));
        printf("ROUTE: pattern='%s', prefix='%s'\n", bdata(route->pattern), bdata(route->prefix));
    } else {
        printf("Match FAILED on '%s' against '%s'.\n", bdata(match), bdata(pattern));
    }

    RouteMap_destroy(map);
    return 0;
error:

    RouteMap_destroy(map);
    return -1;
}
